class Heroes.Player extends Heroes.WorldObject
{
   var d_x;
   var d_y;
   var prevLeft;
   var prevRight;
   var intelligence;
   var prevX;
   var x;
   var prevZ;
   var z;
   var isWaitingToMove;
   var dWalk;
   var dRun;
   var isMoving;
   var graphics;
   var isFacing;
   var floorX;
   var floorH;
   var floorY;
   var state = "sit";
   var jumpLock = false;
   var fireLock = false;
   var jumpCount = -1;
   var fireCount = -1;
   var punchState = 0;
   var facing = "right";
   var facingCounter = 0;
   var wasDirectionDown = false;
   var pA = 0;
   var energy = 100;
   var prevHitTime = 0;
   var isGrabbing = null;
   var isFirstFrame = true;
   var currentMission = 0;
   var canWalk = false;
   var distractCounter = 0;
   var confuseCounter = 0;
   var doRespondToUser = true;
   var dJump = 12;
   var minDM = 30;
   var scrollPointLeft = 235;
   var scrollPointRight = 235;
   var xMin = 30;
   var xMax = 440;
   var repeatHitTimer = 350;
   var facingDirection = 1;
   static var boundBoxWidth = 60;
   static var boundBoxDepth = 10;
   function Player()
   {
      super();
      this.d_x = 0;
      this.d_y = 0;
      Mouse.addListener(this);
      this.prevLeft = this.prevRight = getTimer();
      this.intelligence = Heroes.Tools.getIntelligence();
      this._visible = true;
      this.prevX = this.x;
      this.prevZ = this.z;
      this.face("right");
   }
   function onEnterFrame()
   {
      this.confuseCounter = this.confuseCounter - 1;
      this.distractCounter = this.distractCounter - 1;
      this.facingCounter = this.facingCounter - 1;
      if(this.confuseCounter > 0)
      {
         this._parent._parent._parent.cursor.setState("wonky");
      }
      else
      {
         this._parent._parent._parent.cursor.setState("normal");
      }
      this.prevX = this.x;
      this.prevZ = this.z;
      if(!this.isFirstFrame)
      {
         if(this.currentMission == 0)
         {
            this.incMission();
         }
         if(!this.isWaitingToMove)
         {
            this.respondeToUser();
         }
      }
      this.updateDistraction();
      this.scrollLevel();
      this.updateFloor();
      this.updateFacing();
      this.isFirstFrame = false;
      super.onEnterFrame();
      this._parent.follower.drawLead();
   }
   function onMouseDown()
   {
      if(this.doRespondToUser)
      {
         if(this.isWaitingToMove)
         {
            this.hideMissionBubble();
         }
         else if(this.isAmbulatingUndistracted())
         {
            this.setState("jogging");
            this._parent.follower.playFasterSound();
         }
      }
   }
   function onMouseUp()
   {
      if(this.doRespondToUser)
      {
         this.stopRunning();
      }
   }
   function stopRunning()
   {
      if(this.state == "jogging")
      {
         this.setState("walking");
         this._parent.follower.playSlowerSound();
      }
   }
   function respondeToUser()
   {
      this.dWalk = 4 + this.intelligence / 40;
      this.dRun = this.getRunningSpeed();
      if(this.doRespondToUser)
      {
         var _loc5_ = _root._xmouse - (this._x + this._parent._x);
         var _loc4_ = _root._ymouse - (this._y + this._parent._y);
         var _loc7_ = Math.sqrt(Math.pow(_loc5_,2) + Math.pow(_loc4_,2));
         var _loc3_ = Math.atan2(_loc4_,_loc5_);
         if(this.state != "jogging")
         {
            this.pA = this.updateAngle(this.pA,_loc3_,1);
         }
         else
         {
            this.pA = this.updateAngle(this.pA,_loc3_,0.05);
         }
         var _loc6_ = this.x;
         if(_loc7_ > this.minDM || this.state == "jogging")
         {
            if(this.canWalk)
            {
               if(this.confuseCounter > 0)
               {
                  if(this.state == "jogging")
                  {
                     this.x += this.dRun * Math.cos(this.pA + 1.2);
                     this.z -= this.dRun * Math.sin(this.pA + 1.2);
                  }
                  else
                  {
                     this.x += this.dWalk * Math.cos(_loc3_ + 1.2);
                     this.z -= this.dWalk * Math.sin(_loc3_ + 1.2);
                  }
               }
               else if(this.state == "jogging")
               {
                  this.x += this.dRun * Math.cos(this.pA);
                  this.z -= this.dRun * Math.sin(this.pA);
               }
               else
               {
                  this.x += this.dWalk * Math.cos(this.pA);
                  this.z -= this.dWalk * Math.sin(this.pA);
               }
               this.isMoving = true;
            }
            if(this.state == "sit")
            {
               this.setState("stand");
               this.pA = _loc3_;
            }
         }
         else
         {
            this.isMoving = false;
            if((this.state == "stand" || this.state == "walking") && this.distractCounter <= 0)
            {
               this.setState("sit");
               this._parent.follower.playSound("sit");
            }
         }
         if(this.state != "sit")
         {
            if(this.x < _loc6_)
            {
               this.face("left");
            }
            else
            {
               this.face("right");
            }
         }
      }
   }
   function getRunningSpeed()
   {
      return 10 + this.intelligence / 15;
   }
   function updateAngle(angleFrom, angleTo, amount)
   {
      var _loc2_ = 1.5707963267948966;
      if(angleFrom < - _loc2_ && angleTo > _loc2_)
      {
         angleFrom += 6.283185307179586;
      }
      if(angleFrom > _loc2_ && angleTo < - _loc2_)
      {
         angleFrom -= 6.283185307179586;
      }
      var _loc3_ = angleTo - angleFrom;
      _loc3_ *= amount;
      return angleFrom + _loc3_;
   }
   function stopRespondingToUser()
   {
      this.setState("sit");
      this.doRespondToUser = false;
      trace("stopRespondingToUser doRespondToUser=" + this.doRespondToUser);
   }
   function isAmbulatingUndistracted()
   {
      return (this.state == "stand" || this.state == "distracted" || this.state == "walking") && this.canWalk;
   }
   function updateDistraction()
   {
      if(this.distractCounter > 0 && this.isAmbulatingUndistracted())
      {
         this.setState("distracted");
      }
      if(this.distractCounter <= 0 && this.state == "distracted")
      {
         this.setState("walking");
      }
   }
   function setState(newState)
   {
      if(this.state != newState)
      {
         this.canWalk = false;
         this.state = newState;
         this.graphics.gotoAndPlay(newState);
      }
   }
   function atWalking()
   {
      this.canWalk = true;
   }
   function distract(distractionCost)
   {
      if(this.doRespondToUser)
      {
         if(this.state == "sit")
         {
            this.setState("stand");
         }
         this.intelligence -= distractionCost;
         if(this.intelligence <= 0)
         {
            this.stopRespondingToUser();
            this._parent._parent.outOfBrain.gotoAndStop("show");
            this._parent._parent.timer.stop();
         }
         this.distractCounter = 2;
      }
   }
   function confuse()
   {
      if(this.doRespondToUser)
      {
         this.confuseCounter = 2;
      }
   }
   function face(newDirection)
   {
      if(newDirection != this.facing)
      {
         if(newDirection == "left")
         {
            this.facingDirection = -1;
         }
         else
         {
            this.facingDirection = 1;
         }
         this.facing = newDirection;
      }
   }
   function updateFacing()
   {
      if(this.facingCounter <= 0 && this.facing != this.isFacing)
      {
         this.isFacing = this.facing;
         this.facingCounter = 5;
         §§push(this.gotoAndStop(this.facing + "_" + Heroes.Tools.playerSkin));
      }
   }
   function scrollLevel()
   {
      if(this.state != "levelComplete")
      {
         if(this._x + this._parent._x > this.scrollPointRight)
         {
            this._parent._x = (this._parent._x + this.scrollPointRight - this._x) / 2;
         }
         if(this._x + this._parent._x < this.scrollPointLeft)
         {
            this._parent._x = (this._parent._x + this.scrollPointLeft - this._x) / 2;
         }
         if(this._parent._x > 0)
         {
            this._parent._x = 0;
         }
         if(this._parent._x < - this._parent._parent.levelEnd)
         {
            this._parent._x = - this._parent._parent.levelEnd;
         }
         if(this._x + this._parent._x > this.xMax)
         {
            this.x += this.xMax - (this._x + this._parent._x);
         }
         if(this._x + this._parent._x < this.xMin)
         {
            this.x -= this._x + this._parent._x - this.xMin;
         }
      }
   }
   function updateFloor()
   {
      if(this.isFirstFrame)
      {
         this.floorX = this._parent.floor._x;
         this.floorH = Math.floor(this._parent.floor._height);
         this.floorY = this._parent.floor._y + this._parent.floor.graphics._y;
      }
      this._parent.floor._x = - this._parent._x + this.floorX;
      this._parent.floor.gotoAndStop(Math.floor((- this._parent._x) % this._parent.floor.loopLength));
      this._parent.floor._y = this.floorY - this._parent.floor.graphics._y;
      this._parent.floor._height = this.floorH;
   }
   function getCurrentMission()
   {
      return "mission_" + this.currentMission;
   }
   function incMission()
   {
      if(this.currentMission > 0)
      {
         this._parent._parent.score.addScore(this._parent[this.getCurrentMission()].score);
         var _loc2_ = this.intelligence / 100;
         trace(_loc2_);
         trace(30 * (1 - _loc2_));
         trace(15 * _loc2_);
         this.intelligence += (25 * (1 - _loc2_) + 15 * _loc2_) / 2;
         if(this.intelligence > 100)
         {
            this.intelligence = 100;
         }
      }
      this.currentMission = this.currentMission + 1;
      this._parent._parent.timer.stop();
      if(this._parent[this.getCurrentMission()] != null)
      {
         this.setState("sit");
         this.showMissionBubble();
         this._parent._parent.taskMaster.graphics.gotoAndStop(this._parent[this.getCurrentMission()].mission);
         this._parent._parent.timer.addTime(this._parent[this.getCurrentMission()].time);
      }
      else
      {
         Heroes.Tools.setScore(this._parent._parent.score.getScore());
         Heroes.Tools.setLevelTime(this._parent._parent.timer.getTime());
         Heroes.Tools.setIntelligence(this.intelligence);
         this.setState("levelComplete");
         this._parent.follower.setState("levelComplete");
         this.doRespondToUser = false;
         this.isWaitingToMove = true;
      }
   }
   function showMissionBubble()
   {
      this._parent._parent.speechBubble.gotoAndStop("show");
      this._parent._parent.speechBubble.graphics.gotoAndStop(this._parent[this.getCurrentMission()].mission);
      this.isWaitingToMove = true;
      var _loc2_ = {x:this._parent.follower.speechBubbleTail._x,y:this._parent.follower.speechBubbleTail._y};
      this._parent.follower.localToGlobal(_loc2_);
      this._parent._parent.globalToLocal(_loc2_);
      this._parent._parent.speechBubble._x = _loc2_.x;
      this._parent._parent.speechBubble._y = _loc2_.y;
   }
   function hideMissionBubble()
   {
      this._parent.follower.playSound("letsgo");
      this._parent._parent.speechBubble.gotoAndStop("hide");
      this.isWaitingToMove = false;
      this._parent._parent.timer.start();
   }
   function onCollectBone()
   {
      this.intelligence += 10;
   }
   function onLevelEnd()
   {
      this._parent._parent.play();
   }
   function onAttack(attackBox, power, forceFall, attackerFacing, hitX, hitY)
   {
      if(this.graphics.bodyBox != null)
      {
         if(attackBox.isHitting(this.graphics.bodyBox))
         {
            if(getTimer() - this.prevHitTime > this.repeatHitTimer)
            {
               this.loseEnergy(power,forceFall);
               this._parent._parent.hits.baddyHitPlayer(hitX,hitY);
               if(attackerFacing != null && forceFall)
               {
                  this.face(attackerFacing);
               }
            }
            this.prevHitTime = getTimer();
         }
      }
   }
   function onProjectileAttck(px, py, pz, power)
   {
      if(this.graphics.bodyBox != null)
      {
         if(this.graphics.bodyBox.isTouching(px,py,pz))
         {
            this.loseEnergy(power,false);
            this._parent._parent.hits.baddyHitPlayer(this.get_X(px,pz),this.get_Y(py,pz));
            return true;
         }
      }
      return false;
   }
   function onLevelComplete()
   {
      this._parent._parent.play();
   }
   function setEnergyBar(newEnergy)
   {
      this._parent._parent.goodEnergy.gotoAndStop(int(newEnergy) + 1);
      if(newEnergy > 0)
      {
         this._parent._parent.goodEnergy.frontEnergy.text = newEnergy.toString();
         this._parent._parent.goodEnergy.rearEnergy.text = newEnergy.toString();
      }
      else
      {
         this._parent._parent.goodEnergy.frontEnergy.text = "0";
         this._parent._parent.goodEnergy.rearEnergy.text = "0";
      }
   }
   function loseEnergy(amount, forceFall)
   {
      this.energy -= amount;
      if(this.energy < 0)
      {
         this.energy == 0;
      }
      this.setEnergyBar(this.energy);
      if(this.energy > 0)
      {
         if(forceFall)
         {
            this.setState("fallDown");
         }
      }
      else
      {
         this.setState("dead");
      }
   }
   function onDead()
   {
   }
}
